@@ -1,5 +1,5 @@
 from celescope.tag.__init__ import __ASSAY__
-from celescope.tools.Multi import Multi
+from celescope.tools.multi import Multi
 
 
 class Multi_tag(Multi):
@@ -25,7 +25,6 @@ class Multi_tag(Multi):
         )
         self.process_cmd(cmd, step, sample, m=5, x=1)
 
-
     def analysis_tag(self, sample):
         step = 'analysis_tag'
         tsne_tag_file = f'{self.outdir_dic[sample]["count_tag"]}/{sample}_tsne_tag.tsv'
@@ -37,11 +36,22 @@ class Multi_tag(Multi):
         )
         self.process_cmd(cmd, step, sample, m=5, x=1)
 
+    def split_tag(self, sample):
+        step = 'split_tag'
+        umi_tag_file = f'{self.outdir_dic[sample]["count_tag"]}/{sample}_umi_tag.tsv'
+        cmd_line = self.get_cmd_line(step, sample)
+        cmd = (
+            f'{cmd_line} '
+            f'--match_dir {self.col4_dict[sample]} '
+            f'--umi_tag_file {umi_tag_file} '
+        )
+        self.process_cmd(cmd, step, sample, m=5, x=1)
+
 
 def main():
     multi = Multi_tag(__ASSAY__)
     multi.run()
 
+
 if __name__ == '__main__':
     main()
-
