@@ -5,8 +5,9 @@ import os
 from collections import defaultdict
 
 import celescope
-from celescope.tools.utils import find_assay_init, find_step_module
+import celescope.tools.utils as utils
 from celescope.celescope import ArgFormatter
+from celescope.__init__ import HELP_DICT
 
 TOOLS_DIR = os.path.dirname(celescope.tools.__file__)
 
@@ -15,40 +16,45 @@ class Multi():
 
     def __init__(self, assay):
         self.__ASSAY__ = assay
-        init_module = find_assay_init(assay)
+        init_module = utils.find_assay_init(assay)
         self.__STEPS__ = init_module.__STEPS__
         self.__CONDA__ = os.path.basename(os.environ['CONDA_DEFAULT_ENV'])
         self.__APP__ = 'celescope'
-        self.col4_default = None
-        self.last_step = ''
-        self.args = None
         self.steps_not_run = ['mkref']
-        
+
         # remove
         for step in self.steps_not_run:
             if step in self.__STEPS__:
                 self.__STEPS__.remove(step)
 
-        # parse_args
+        # add args
+        self.parser = None
         self.common_args()
         self.step_args()
-        self.args = self.parser.parse_args()
-        if self.args.gzip:
-            self.fq_suffix = ".gz"
-        else:
-            self.fq_suffix = ""
-        if self.args.steps_run == 'all':
-            self.steps_run = self.__STEPS__
-        elif self.args.steps_run:
-            self.steps_run = self.args.steps_run.strip().split(',')
 
-        self.prepare()
+        # set
+        self.args = None
+        self.col4_default = None
+        self.last_step = ''
+        self.fq_suffix = ""
+        self.steps_run = self.__STEPS__
+        self.fq_dict = {}
+        self.col4_dict = {}
+        self.col5_dict = {}
+        self.logdir = None
+
+        self.sjm_cmd = ''
+        self.sjm_order = ''
+        self.shell_dict = defaultdict(str)
+
+        self.outdir_dic = {}
+
 
     def common_args(self):
         readme = f'{self.__ASSAY__} multi-samples'
-        parser = argparse.ArgumentParser(readme, 
-            formatter_class=ArgFormatter,
-            conflict_handler='resolve')
+        parser = argparse.ArgumentParser(readme,
+                                         formatter_class=ArgFormatter,
+                                         conflict_handler='resolve')
         parser.add_argument('--mod', help='mod, sjm or shell', choices=['sjm', 'shell'], default='sjm')
         parser.add_argument(
             '--mapfile',
@@ -57,28 +63,30 @@ class Multi():
                 1st col: LibName;
                 2nd col: DataDir;
                 3rd col: SampleName;
-                4th col: Cell number or match_dir, optional;
+                4th col: optional;
             ''',
             required=True)
         parser.add_argument('--rm_files', action='store_true', help='remove redundant fq.gz and bam after running')
-        parser.add_argument('--steps_run', help='steps to run', default='all')
+        parser.add_argument('--steps_run', help='Steps to run. Multiple Steps are separated by comma.', default='all')
         # sub_program parser do not have
-        parser.add_argument('--outdir', help='output dir', default="./")
-        parser.add_argument('--debug', help='debug or not', action='store_true')
-        parser.add_argument('--thread', help='thread', default=4)
+        parser.add_argument('--outdir', help='Output directory.', default="./")
+        parser.add_argument('--thread', help=HELP_DICT['thread'], default=4)
+        parser.add_argument('--debug', help=HELP_DICT['debug'], action='store_true')
         self.parser = parser
         return parser
 
     def step_args(self):
         for step in self.__STEPS__:
-            step_module = find_step_module(self.__ASSAY__, step)
+            step_module = utils.find_step_module(self.__ASSAY__, step)
             func_opts = getattr(step_module, f"get_opts_{step}")
             func_opts(self.parser, sub_program=False)
 
     @staticmethod
-    def parse_map_col4(mapfile, default_val):
+    @utils.add_log
+    def parse_mapfile(mapfile, default_val):
         fq_dict = defaultdict(list)
         col4_dict = {}
+        col5_dict = {}
         with open(mapfile) as fh:
             for line in fh:
                 line = line.strip()
@@ -86,7 +94,7 @@ class Multi():
                     continue
                 line_split = line.split()
                 library_id, library_path, sample_name = line_split[:3]
-                if len(line_split) == 4:
+                if len(line_split) >= 4:
                     col4 = line_split[3]
                 else:
                     col4 = default_val
@@ -98,7 +106,8 @@ class Multi():
                 else:
                     fq_dict[sample_name] = [[fq1], [fq2]]
                     col4_dict[sample_name] = col4
-
+                if len(line_split) == 5:
+                    col5_dict[sample_name] = line_split[4]
 
         for sample_name in fq_dict:
             fq_dict[sample_name][0] = ",".join(fq_dict[sample_name][0])
@@ -106,7 +115,7 @@ class Multi():
 
         if not fq_dict:
             raise Exception('empty mapfile!')
-        return fq_dict, col4_dict
+        return fq_dict, col4_dict, col5_dict
 
     def link_data(self):
         raw_dir = f'{self.args.outdir}/data_give/rawdata'
@@ -119,25 +128,25 @@ class Multi():
 
     def prepare(self):
         """
-        parse_mapfile, link data, make log dir, init script variables, init outdir_dic
+        parse_mapfile, make log dir, init script variables, init outdir_dic
         """
-        # parse_mapfile
-        self.fq_dict, self.col4_dict = self.parse_map_col4(self.args.mapfile, self.col4_default)
+        self.args = self.parser.parse_args()
 
-        # link
-        self.link_data()
+        if self.args.gzip:
+            self.fq_suffix = ".gz"
+        if self.args.steps_run != 'all':
+            self.steps_run = self.args.steps_run.strip().split(',')
+
+        self.logdir = self.args.outdir + '/log'
+        self.sjm_cmd = f'log_dir {self.logdir}\n'
+
+        # parse_mapfile
+        self.fq_dict, self.col4_dict, self.col5_dict = self.parse_mapfile(self.args.mapfile, self.col4_default)
 
         # mk log dir
-        self.logdir = self.args.outdir + '/log'
-        os.system('mkdir -p %s' % (self.logdir))
+        if self.args.mod == 'sjm':
+            os.system('mkdir -p %s' % (self.logdir))
 
-        # script init
-        self.sjm_cmd = 'log_dir %s\n' % (self.logdir)
-        self.sjm_order = ''
-        self.shell_dict = defaultdict(str)
-
-        # outdir dict
-        self.outdir_dic = {}
         for sample in self.fq_dict:
             self.outdir_dic[sample] = {}
             index = 0
@@ -145,7 +154,7 @@ class Multi():
                 step_outdir = f"{self.args.outdir}/{sample}/{index:02d}.{step}"
                 self.outdir_dic[sample].update({step: step_outdir})
                 index += 1
-    
+
     def generate_cmd(self, cmd, step, sample, m=1, x=1):
         if sample:
             sample = "_" + sample
@@ -165,7 +174,7 @@ job_end
         self.last_step = step
 
     def parse_step_args(self, step):
-        step_module = find_step_module(self.__ASSAY__, step)
+        step_module = utils.find_step_module(self.__ASSAY__, step)
         func_opts = getattr(step_module, f"get_opts_{step}")
         step_parser = argparse.ArgumentParser(step_module)
         func_opts(step_parser, sub_program=False)
@@ -186,7 +195,7 @@ job_end
             f'--thread {self.args.thread} '
         )
         cmd_line = step_prefix
-        if self.args.debug or self.__CONDA__ == "celescope_RD":
+        if self.args.debug:
             cmd_line += " --debug "
         for arg in args_dict:
             if args_dict[arg] is False:
@@ -197,7 +206,7 @@ job_end
                 if args_dict[arg]:
                     matches = [' ', '-']
                     arg_string = str(args_dict[arg])
-                    if any(char in arg_string for char in matches): # need quote
+                    if any(char in arg_string for char in matches):  # need quote
                         cmd_line += f'--{arg} "{arg_string}" '
                     else:
                         cmd_line += f'--{arg} {arg_string} '
@@ -213,7 +222,7 @@ job_end
             f'--fq1 {arr[0]} '
         )
         self.process_cmd(cmd, step, sample, m=1, x=1)
-    
+
     def barcode(self, sample):
         step = "barcode"
         arr = self.fq_dict[sample]
@@ -302,7 +311,7 @@ job_end
                     ) from attr_not_exist
                 method_to_call(sample)
 
-    def merge_report(self):    
+    def merge_report(self):
         step = "merge_report"
         steps_str = ",".join(self.__STEPS__)
         samples = ','.join(self.fq_dict.keys())
@@ -327,10 +336,11 @@ job_end
             os.system('mkdir -p ./shell/')
             for sample in self.shell_dict:
                 with open(f'./shell/{sample}.sh', 'w') as f:
-                    f.write("set -e\n")
+                    f.write("set -eo pipefail\n")
                     f.write(self.shell_dict[sample])
 
     def run(self):
+        self.prepare()
         self.run_steps()
         self.end()
 
@@ -340,7 +350,7 @@ def get_read(library_id, library_path, read='1'):
     fq_list = ['fq', 'fastq']
     suffix_list = ["", ".gz"]
     read_pattern_list = [
-        f'{library_path}/*{library_id}*{read}.{fq_str}{suffix}'
+        f'{library_path}/{library_id}*{read}.{fq_str}{suffix}'
         for read in read1_list
         for fq_str in fq_list
         for suffix in suffix_list
