@@ -1,28 +1,27 @@
+import argparse
+import glob
+import gzip
+import importlib
+import itertools
+import json
 import logging
 import os
-import glob
-import sys
 import re
-import subprocess
-import importlib
 import resource
+import subprocess
 import time
-import argparse
-import gzip
-import itertools
+from collections import Counter, defaultdict
 from datetime import timedelta
-from collections import defaultdict
 from functools import wraps
-from collections import Counter
-import json
 
-import pandas as pd
 import numpy as np
+import pandas as pd
 import pysam
 import xopen
 
 import celescope.tools
 from celescope.tools.__init__ import __PATTERN_DICT__
+from celescope.capture_virus.otsu import array2hist, makePlot, threshold_otsu
 
 tools_dir = os.path.dirname(celescope.tools.__file__)
 
@@ -31,20 +30,22 @@ def add_log(func):
     '''
     logging start and done.
     '''
-    logging.basicConfig(
-        level=logging.INFO,
-        stream=sys.stdout,
-        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
-    )
+    logFormatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
+
     module = func.__module__
     name = func.__name__
     logger_name = f'{module}.{name}'
     logger = logging.getLogger(logger_name)
+    logger.setLevel(logging.INFO)
+
+    consoleHandler = logging.StreamHandler()
+    consoleHandler.setFormatter(logFormatter)
+    logger.addHandler(consoleHandler)
 
     @wraps(func)
     def wrapper(*args, **kwargs):
         if args and hasattr(args[0], 'debug') and args[0].debug:
-            logger.setLevel(10) # debug
+            logger.setLevel(10)  # debug
 
         logger.info('start...')
         start = time.time()
@@ -59,10 +60,10 @@ def add_log(func):
 
 
 def using(point=""):
-    usage=resource.getrusage(resource.RUSAGE_SELF)
+    usage = resource.getrusage(resource.RUSAGE_SELF)
     return '''%s: usertime=%s systime=%s mem=%s mb
-        '''%(point,usage[0],usage[1],
-                usage[2]/1024.0)
+        ''' % (point, usage[0], usage[1],
+               usage[2]/1024.0)
 
 
 def add_mem(func):
@@ -98,27 +99,6 @@ def arg_str(arg, arg_name):
     return ''
 
 
-def read_barcode_file(match_dir, return_file=False):
-    '''
-    multi version compatible
-    '''
-    match_barcode_file1 = glob.glob(
-        f"{match_dir}/*count*/*_cellbarcode.tsv")
-    match_barcode_file2 = glob.glob(
-        f"{match_dir}/*count*/*matrix_10X/*_cellbarcode.tsv")
-    match_barcode_file3 = glob.glob(
-        f"{match_dir}/*count*/*matrix_10X/*barcodes.tsv")
-    match_barcode_file = (
-        match_barcode_file1 +
-        match_barcode_file2 +
-        match_barcode_file3)[0]
-    match_barcode, cell_total = read_one_col(match_barcode_file)
-    match_barcode = set(match_barcode)
-    if return_file:
-        return match_barcode, (cell_total, match_barcode_file)
-    return match_barcode, cell_total
-
-
 def format_stat(count, total_count):
     percent = round(count / total_count * 100, 2)
     string = f'{format_number(count)}({percent}%)'
@@ -195,15 +175,25 @@ def link_data(outdir, fq_dict):
             fh.write('ln -sf %s %s\n' % (arr[1], s + '_2.fq.gz'))
 
 
-def generic_open(file_name, mode='rt'):
+def generic_open(file_name, *args, **kwargs):
     if file_name.endswith('.gz'):
-        file_obj = gzip.open(file_name, mode)
+        file_obj = gzip.open(file_name, *args, **kwargs)
     else:
-        file_obj = open(file_name, mode)
+        file_obj = open(file_name, *args, **kwargs)
     return file_obj
 
 
-def gene_convert(gtf_file):
+@add_log
+def get_id_name_dict(gtf_file):
+    """
+    get gene_id:gene_name from gtf file
+        - one gene_name with multiple gene_id: "_{count}" will be added to gene_name.
+        - one gene_id with multiple gene_name: error.
+        - duplicated (gene_name, gene_id): ignore duplicated records and print a warning.
+
+    Returns:
+        {gene_id: gene_name} dict
+    """
 
     gene_id_pattern = re.compile(r'gene_id "(\S+)";')
     gene_name_pattern = re.compile(r'gene_name "(\S+)"')
@@ -221,20 +211,34 @@ def gene_convert(gtf_file):
                 gene_id = gene_id_pattern.findall(attributes)[-1]
                 gene_names = gene_name_pattern.findall(attributes)
                 if not gene_names:
-                    gene_name = gene_id 
+                    gene_name = gene_id
                 else:
                     gene_name = gene_names[-1]
                 c[gene_name] += 1
                 if c[gene_name] > 1:
-                    gene_name = f'{gene_name}_{c[gene_name]}'
+                    if gene_id in id_name:
+                        assert id_name[gene_id] == gene_name, (
+                            'one gene_id with multiple gene_name '
+                            f'gene_id: {gene_id}, '
+                            f'gene_name this line: {gene_name}'
+                            f'gene_name previous line: {id_name[gene_id]}'
+                        )
+                        get_id_name_dict.logger.warning(
+                            'duplicated (gene_id, gene_name)'
+                            f'gene_id: {gene_id}, '
+                            f'gene_name {gene_name}'
+                        )
+                        c[gene_name] -= 1
+                    else:
+                        gene_name = f'{gene_name}_{c[gene_name]}'
                 id_name[gene_id] = gene_name
     return id_name
 
 
 @add_log
 def process_read(
-    read2_file, pattern_dict, barcode_dict, linker_dict,
-    barcode_length, linker_length):
+        read2_file, pattern_dict, barcode_dict, linker_dict,
+        barcode_length, linker_length):
 
     # if valid, return (True)
     metrics = defaultdict(int)
@@ -264,8 +268,8 @@ def process_read(
                 if miss_length > 2:
                     metrics['Reads Unmapped too Short'] += 1
                     continue
-                seq_barcode = seq_barcode + "A" * miss_length                    
-        
+                seq_barcode = seq_barcode + "A" * miss_length
+
         # check linker
         if linker_length != 0:
             valid_linker = False
@@ -275,7 +279,7 @@ def process_read(
                     break
         else:
             valid_linker = True
-            
+
         if not valid_linker:
             metrics['Reads Unmapped Invalid Linker'] += 1
             continue
@@ -365,7 +369,7 @@ def gen_stat(df, stat_file):
         value = f'{format_number(count)}({round(percent * 100, 2)}%)'
         return value
 
-    df.loc[:,'value'] = df.loc[:,'count']
+    df.loc[:, 'value'] = df.loc[:, 'count']
     df.loc[~df['total_count'].isna(), 'value'] = df.loc[~df['total_count'].isna(), :].apply(
         add_percent, axis=1
     )
@@ -381,9 +385,9 @@ def get_read(library_id, library_path, read='1'):
     fq_list = ['fq', 'fastq']
     suffix_list = ["", ".gz"]
     read_pattern_list = [
-        f'{library_path}/*{library_id}*{read}.{fq_str}{suffix}' 
-        for read in read1_list 
-        for fq_str in fq_list 
+        f'{library_path}/*{library_id}*{read}.{fq_str}{suffix}'
+        for read in read1_list
+        for fq_str in fq_list
         for suffix in suffix_list
     ]
     fq_list = [glob.glob(read1_pattern) for read1_pattern in read_pattern_list]
@@ -412,76 +416,6 @@ def get_fq(library_id, library_path):
     return fq1, fq2
 
 
-def parse_map_col4(mapfile, default_val):
-    fq_dict = defaultdict(list)
-    col4_dict = defaultdict(list)
-    with open(mapfile) as fh:
-        for line in fh:
-            line = line.strip()
-            if not line:
-                continue
-            if line.startswith('#'):
-                continue
-            tmp = line.split()
-            library_id = tmp[0]
-            library_path = tmp[1]
-            sample_name = tmp[2]
-            if len(tmp) == 4:
-                col4 = tmp[3]
-            else:
-                col4 = default_val
-            fq1, fq2 = get_fq(library_id, library_path)
-
-            if sample_name in fq_dict:
-                fq_dict[sample_name][0].append(fq1)
-                fq_dict[sample_name][1].append(fq2)
-            else:
-                fq_dict[sample_name] = [[fq1], [fq2]]
-            if col4 and col4 != default_val:
-                col4_dict[sample_name] = col4
-
-    for sample_name in fq_dict:
-        fq_dict[sample_name][0] = ",".join(fq_dict[sample_name][0])
-        fq_dict[sample_name][1] = ",".join(fq_dict[sample_name][1])
-
-    if not fq_dict:
-        raise Exception('empty mapfile!')
-    return fq_dict, col4_dict
-
-
-def generate_sjm(cmd, name, conda, m=1, x=1):
-    res_cmd = f'''
-job_begin
-    name {name}
-    sched_options -w n -cwd -V -l vf={m}g,p={x}
-    cmd source activate {conda}; {cmd}
-job_end
-'''
-
-    return res_cmd
-
-
-def merge_report(
-    fq_dict, steps, last_step, sjm_cmd,
-    sjm_order, logdir, conda, outdir, rm_files):    
-    step = "merge_report"
-    steps_str = ",".join(steps)
-    samples = ','.join(fq_dict.keys())
-    app = tools_dir + '/merge_table.py'
-    cmd = (
-        f'python {app} --samples {samples} '
-        f'--steps {steps_str} --outdir {outdir}'
-    )
-    if rm_files:
-        cmd += ' --rm_files'
-    sjm_cmd += generate_sjm(cmd, 'merge_report', conda)
-    for sample in fq_dict:
-        sjm_order += f'order {step} after {last_step}_{sample}\n'
-    with open(logdir + '/sjm.job', 'w') as fh:
-        fh.write(sjm_cmd + '\n')
-        fh.write(sjm_order)
-
-
 def format_number(number: int) -> str:
     return format(number, ",")
 
@@ -499,42 +433,6 @@ def format_ratios(ratios: dict):
         ratios[key] = round(ratios[key] * 100, 2)
 
 
-@add_log
-def glob_genomeDir(genomeDir, fa=False):
-    refFlat = glob.glob(genomeDir + "/*.refFlat")
-    if (len(refFlat) > 1):
-        sys.exit("ERROR: Multiple refFlat file in " + genomeDir)
-    elif (len(refFlat) == 0):
-        sys.exit("ERROR: refFlat file not found in " + genomeDir)
-    else:
-        refFlat = refFlat[0]
-        glob_genomeDir.logger.info("refFlat file found: " + refFlat)
-
-    gtf = glob.glob(genomeDir + "/*.gtf")
-    if (len(gtf) == 0):
-        sys.exit("ERROR: gtf file not found in " + genomeDir)
-    elif (len(gtf) > 1):
-        gtf = glob.glob(genomeDir + "/*.chr.gtf")
-        if (len(gtf) == 0):
-            sys.exit("ERROR: No chr gtf file in "+ genomeDir)
-        if (len(gtf) > 1):
-            sys.exit("ERROR: Multiple gtf file in " + genomeDir)
-        else:
-            gtf = gtf[0]
-            glob_genomeDir.logger.info("chr gtf file found: " + gtf)
-    else:
-        gtf = gtf[0]
-        glob_genomeDir.logger.info("gtf file found: " + gtf)
-    
-    if fa:
-        fasta = glob.glob(genomeDir + "/*.fa") + glob.glob(genomeDir + "/*.fasta")
-        if len(fasta) > 1:
-            sys.exit("ERROR: Multiple fasta file in " + genomeDir)
-        fasta = fasta[0]
-        return refFlat, gtf, fasta
-    return refFlat, gtf, None
-
-
 def get_slope(x, y, window=200, step=10):
     assert len(x) == len(y)
     start = 0
@@ -557,98 +455,6 @@ def genDict(dim=3, valType=int):
     else:
         return defaultdict(lambda: genDict(dim - 1, valType=valType))
 
-'''
-@add_log
-def downsample(bam, barcodes, percent):
-    """
-    calculate median_geneNum and saturation based on a given percent of reads
-
-    Args:
-        bam - bam file
-        barcodes(set) - validated barcodes
-        percent(float) - percent of reads in bam file.
-
-    Returns:
-        percent(float) - input percent
-        median_geneNum(int) - median gene number
-        saturation(float) - sequencing saturation
-
-    Description:
-        Sequencing Saturation = 1 - n_deduped_reads / n_reads.
-        n_deduped_reads = Number of unique (valid cell-barcode, valid UMI, gene) combinations among confidently mapped reads.
-        n_reads = Total number of confidently mapped, valid cell-barcode, valid UMI reads.
-    """
-    logging.info('working' + str(percent))
-    cmd = ['samtools', 'view', '-s', str(percent), bam]
-    p1 = subprocess.Popen(cmd, stdout=subprocess.PIPE)
-    # nesting defaultdicts in an arbitrary depth
-    readDict = genDict()
-
-    n_reads = 0
-    while True:
-        line = p1.stdout.readline()
-        if not line.strip():
-            break
-        tmp = line.strip().split()
-        if tmp[-1].startswith('XT:Z:'):
-            geneID = tmp[-1].replace('XT:Z:', '')
-            cell_barcode, umi = tmp[0].split('_')[0:2]
-            # filter invalid barcode
-            if cell_barcode in barcodes:
-                n_reads += 1
-                readDict[cell_barcode][umi][geneID] += 1
-    p1.stdout.close()
-
-    geneNum_list = []
-    n_deduped_reads = 0
-    for cell_barcode in readDict:
-        genes = set()
-        for umi in readDict[cell_barcode]:
-            for geneID in readDict[cell_barcode][umi]:
-                genes.add(geneID)
-                if readDict[cell_barcode][umi][geneID] == 1:
-                    n_deduped_reads += 1
-        geneNum_list.append(len(genes))
-
-    median_geneNum = np.median(geneNum_list) if geneNum_list else 0
-    saturation = (1 - float(n_deduped_reads) / n_reads) * 100
-
-    return "%.2f\t%.2f\t%.2f\n" % (
-        percent, median_geneNum, saturation), saturation
-'''
-
-
-def cluster_tsne_list(tsne_df):
-    """
-    tSNE_1	tSNE_2	cluster Gene_Counts
-    return data list
-    """
-    sum_df = tsne_df.groupby(["cluster"]).agg("count").iloc[:, 0]
-    percent_df = sum_df.transform(lambda x: round(x / sum(x) * 100, 2))
-    res = []
-    for cluster in sorted(tsne_df.cluster.unique()):
-        sub_df = tsne_df[tsne_df.cluster == cluster]
-        name = "cluster {cluster}({percent}%)".format(
-            cluster=cluster, percent=percent_df[cluster])
-        tSNE_1 = list(sub_df.tSNE_1)
-        tSNE_2 = list(sub_df.tSNE_2)
-        res.append({"name": name, "tSNE_1": tSNE_1, "tSNE_2": tSNE_2})
-    return res
-
-
-def marker_table(marker_df):
-    """
-    return html code
-    """
-    marker_df = marker_df.loc[:, ["cluster", "gene",
-                                  "avg_log2FC", "pct.1", "pct.2", "p_val_adj"]]
-    marker_gene_table = marker_df.to_html(
-        escape=False,
-        index=False,
-        table_id="marker_gene_table",
-        justify="center")
-    return marker_gene_table
-
 
 def report_prepare(outdir, **kwargs):
     json_file = outdir + '/../.data.json'
@@ -666,7 +472,7 @@ def report_prepare(outdir, **kwargs):
         json.dump(data, fh)
 
 
-def parse_vcf(vcf_file, cols=('chrom', 'pos', 'alleles'), infos=('VID','CID')):
+def parse_vcf(vcf_file, cols=('chrom', 'pos', 'alleles'), infos=('VID', 'CID')):
     vcf = pysam.VariantFile(vcf_file)
     df = pd.DataFrame(columns=[col.capitalize() for col in cols] + infos)
     rec_dict = {}
@@ -676,7 +482,7 @@ def parse_vcf(vcf_file, cols=('chrom', 'pos', 'alleles'), infos=('VID','CID')):
             rec_dict[col.capitalize()] = getattr(rec, col)
             if col == 'alleles':
                 rec_dict['Alleles'] = '-'.join(rec_dict['Alleles'])
-                
+
         for info in infos:
             rec_dict[info] = rec.info[info]
 
@@ -686,12 +492,13 @@ def parse_vcf(vcf_file, cols=('chrom', 'pos', 'alleles'), infos=('VID','CID')):
         rec_dict['GT'] = '/'.join(rec_dict['GT'])
         '''
 
-        df = df.append(pd.Series(rec_dict),ignore_index=True)
+        df = df.append(pd.Series(rec_dict), ignore_index=True)
+    vcf.close()
     return df
 
 
 def parse_annovar(annovar_file):
-    df = pd.DataFrame(columns=['Gene','mRNA', 'Protein', 'COSMIC'])
+    df = pd.DataFrame(columns=['Gene', 'mRNA', 'Protein', 'COSMIC'])
     with open(annovar_file, 'rt') as f:
         index = 0
         for line in f:
@@ -721,7 +528,7 @@ def parse_annovar(annovar_file):
                     if change_attr.startswith('p.'):
                         protein = change_attr.strip('p.')
                 if not (mRNA, protein) in change_list:
-                    change_list.append((mRNA, protein)) 
+                    change_list.append((mRNA, protein))
             combine = [','.join(item) for item in list(zip(*change_list))]
             mRNA = combine[0]
             protein = combine[1]
@@ -734,11 +541,38 @@ def parse_annovar(annovar_file):
     return df
 
 
+def read_barcode_file(match_dir, return_file=False):
+    '''
+    multi version compatible
+    '''
+    match_barcode_file1 = glob.glob(
+        f"{match_dir}/*count*/*_cellbarcode.tsv")
+    match_barcode_file2 = glob.glob(
+        f"{match_dir}/*count*/*matrix_10X/*_cellbarcode.tsv")
+    match_barcode_file3 = glob.glob(
+        f"{match_dir}/*count*/*matrix_10X/*barcodes.tsv")
+    match_barcode_file = (
+        match_barcode_file1 +
+        match_barcode_file2 +
+        match_barcode_file3)[0]
+    match_barcode, cell_total = read_one_col(match_barcode_file)
+    if return_file:
+        return match_barcode, (cell_total, match_barcode_file)
+    return match_barcode, cell_total
+
+
+def get_barcodes_from_matrix_dir(matrix_dir):
+    barcodes_file = f'{matrix_dir}/barcodes.tsv'
+    match_barcode, _cell_total = read_one_col(barcodes_file)
+    return match_barcode
+
+
 def parse_match_dir(match_dir):
     match_dict = {}
     match_barcode, cell_total = read_barcode_file(match_dir)
     match_dict['match_barcode'] = match_barcode
     match_dict['cell_total'] = cell_total
+    match_dict['matrix_dir'] = glob.glob(f'{match_dir}/*count*/*matrix_10X')[0]
     match_dict['tsne_coord'] = glob.glob(f'{match_dir}/*analysis*/*tsne_coord.tsv')[0]
     match_dict['markers'] = glob.glob(f'{match_dir}/*analysis*/*markers.tsv')[0]
     try:
@@ -830,3 +664,52 @@ def find_step_module(assay, step):
             step_module = importlib.import_module(f"{module_path}.{step}")
 
     return step_module
+
+
+def find_step_module_with_folder(assay, step):
+    init_module = find_assay_init(assay)
+    folder = ""
+    try:
+        step_module = importlib.import_module(f"celescope.{assay}.{step}")
+        folder = assay
+    except ModuleNotFoundError:
+        try:
+            step_module = importlib.import_module(f"celescope.tools.{step}")
+            folder = 'tools'
+        except ModuleNotFoundError:
+            module_path = init_module.IMPORT_DICT[step]
+            step_module = importlib.import_module(f"{module_path}.{step}")
+            folder = module_path.split('.')[1]
+
+    return step_module, folder
+
+
+def sort_bam(input_bam, output_bam, threads=1):
+    cmd = (
+        f'samtools sort {input_bam} '
+        f'-o {output_bam} '
+        f'--threads {threads} '
+    )
+    subprocess.check_call(cmd, shell=True)
+
+
+def index_bam(input_bam):
+    cmd = f"samtools index {input_bam}"
+    subprocess.check_call(cmd, shell=True)
+
+
+def check_mkdir(dir_name):
+    if not os.path.exists(dir_name):
+        os.system(f"mkdir -p {dir_name}")
+
+
+def otsu_min_support_read(array, otsu_plot):
+    """
+    get otsu threshold and plot
+    """
+    array = np.log10(array)
+    hist = array2hist(array)
+    thresh = threshold_otsu(hist)
+    makePlot(hist, thresh, otsu_plot)
+    threshold = round(10 ** thresh,1)
+    return threshold
